﻿using System;
using System.Data;
using System.Windows.Forms;
using System.Data.SqlClient;
using Microsoft.VisualBasic.FileIO;
using System.Diagnostics;
using System.IO;
using System.Media;

namespace TestTVP
{
    public partial class InitialForm : Form
    {
        const String strcColCount = "47Col";
        const String strcFileDir = "D:\\StudiesData\\Test\\TestTVP\\" + strcColCount + "\\";
        StreamWriter logFile; // = new StreamWriter(strcFileDir + @"TvpTest.log", false);
//        StreamWriter logFile = new StreamWriter(strcFileDir + @"TvpTest.log", false);
        int recordCount = 0;
        String strDesc;

        public InitialForm()
        {
            InitializeComponent();
            button1.Text = "Ready";
            ClearLabels();

        }

        private void InitialForm_Closing(Object sender, FormClosingEventArgs e)
        {
            //logFile.Close();
            //logFile.Dispose();
        }

        private void UpdateControls()
        {
            button1.Invalidate();
            button1.Update();
            button1.Refresh();
            Application.DoEvents();
        }

        private void ClearLabels()
        {
            lblStartTime.Text = String.Empty;
            lblEndTime.Text = String.Empty;
            lblElapsed.Text = String.Empty;
            lblFilename.Text = String.Empty;
            lblTotalElapsed.Text = String.Empty;
            lblFullStartTime.Text = String.Empty;
            lblFullEndTime.Text = String.Empty;
            lblFullElapsedTime.Text = String.Empty;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            DateTime datStart, datEnd;
            Stopwatch swInterval, swTotal;
            double duration, fullduration = 0;
            String strFilenameWithPrefix;
            String[] strAllFilenames = { "0000100", "0002000","0004000", "0008000", "0010000", "0012000", "0014000", "0016000", "0018000", "0022000", "0025000","0050000", "0075000", "0100000", "0125000", "0150000", "0175000", "0200000", "0250000",
                                            "0300000", "0400000", "0500000", "0600000", "0700000", "0800000", "0900000", "1000000", "1250000", "1500000", "1750000", "2000000" }; // , "3240000"  // do not go to 2.5 m because datatable will crash program

            String[] strSmallTestFilenames = { "0000100", "0002000", "0004000", "0008000", "0010000", "0012000", "0014000", "0016000", "0018000", "0022000", "0025000", "0050000", "0075000", "0100000", "0125000", "0150000", "0175000", "0200000", "0250000" };

            String[] strSmallestTestFilenames = { "0000100", "0002000", "0004000", "0008000", "0010000", "0012000", "0014000", "0016000", "0018000", "0022000", "0025000" };

            String[] strOnlyOneTestFilenames = { "0025000" };

            String strFileSuffix = "";
            if (rbStringDataTableNoMax.Checked)
            {
                strFileSuffix = " Full String nomax " + (rbTraceOff.Checked ? " trc off" : (rbTraceOn.Checked ? " trc on" : " xevt on")) + (rbStmtOn.Checked ? " stmt on" : " stmt off");
                strDesc = "String,=na()," + (rbTraceOff.Checked ? "trcoff" : (rbTraceOn.Checked ? "trcon" : "xevton")) + "," + (rbStmtOn.Checked ? "stmton" : "stmtoff");
            }
            else if (rbStringDataTableWithMax.Checked)
            {
                strFileSuffix = " Full String max " + nudMaxLength.Value.ToString() + (rbTraceOff.Checked ? " trc off" : (rbTraceOn.Checked ? " trc on" : " xevt on")) + (rbStmtOn.Checked ? " stmt on" : " stmt off");
                strDesc = "String," + nudMaxLength.Value.ToString() + "," + (rbTraceOff.Checked ? "trcoff" : (rbTraceOn.Checked ? "trcon" : "xevton")) + "," + (rbStmtOn.Checked ? "stmton" : "stmtoff");
            }
            else if (rbTypedDataTable.Checked)
            {
                strFileSuffix = " Full Map " + (rbTraceOff.Checked ? " trc off" : (rbTraceOn.Checked ? " trc on" : " xevt on")) + (rbStmtOn.Checked ? " stmt on" : " stmt off");
                strDesc = "Map,=na()," + (rbTraceOff.Checked ? "trcoff" : (rbTraceOn.Checked ? "trcon" : "xevton")) + "," + (rbStmtOn.Checked ? "stmton" : "stmtoff");
            }

            logFile = new StreamWriter(strcFileDir + @"TvpTest" + strFileSuffix + @".log", false);
            button1.Text = "Running";

            lblFullStartTime.Text = DateTime.Now.ToLongTimeString();
            swTotal = Stopwatch.StartNew();
            for (int iteration = 0; iteration < nudNumIterations.Value; iteration++)
            {
                lblIteration.Text = "Iteration #" + (iteration+1).ToString();
                foreach (string strFilename in strAllFilenames)
                {
                    strFilenameWithPrefix = strcColCount + "-" + strFilename;
                    swInterval = Stopwatch.StartNew();
                    datStart = DateTime.Now;
                    //ClearLabels();
                    lblTotalElapsed.Text = String.Empty;
                    lblStartTime.Text = datStart.ToLongTimeString();
                    lblEndTime.Text = String.Empty; // clear it out so we don't see the end being equal to the start of the next file
                    UpdateControls();

                    if (rbStringDataTableNoMax.Checked)
                    {
                        LoadFileUsingStringDataTable(strFilenameWithPrefix, strFilename);
                    }
                    else if (rbStringDataTableWithMax.Checked)
                    {
                        LoadFileUsingFieldTypesDataTable(strFilenameWithPrefix, strFilename);
                    }
                    else if (rbTypedDataTable.Checked)
                    {
                        LoadFileUsingStringMaxLenDataTable(strFilenameWithPrefix, strFilename, (int)nudMaxLength.Value);
                    }

                    datEnd = DateTime.Now;
                    lblEndTime.Text = datEnd.ToLongTimeString();
                    UpdateControls();
                    swInterval.Stop();
                    duration = swInterval.Elapsed.TotalMilliseconds / 1000.0;
                    fullduration += duration;
                    lblElapsed.Text = String.Format("{0:#,##0.000} seconds", (datEnd - datStart).TotalMilliseconds / 1000.0);
                    logFile.WriteLine("{0},{1},{2},{3},{4}", strDesc, recordCount, "Total", duration.ToString("#0.000"), recordCount / duration); //duration.ToString("HH:mm:ss.ffffff"));
                    UpdateControls();
                } // strFilename
                lblFullElapsedTime.Text = fullduration.ToString("#0.000") + " seconds";
                lblEndTime.Text = DateTime.Now.ToLongTimeString();
                SystemSounds.Hand.Play();
            } // iteration loop
            swTotal.Stop();

            logFile.Close();
            logFile.Dispose();

            duration = swTotal.Elapsed.TotalMilliseconds / 1000.0;
            lblTotalElapsed.Text = "Total: " + duration.ToString("#,##0.000") + " seconds";
            button1.Text = "Ready";
            SystemSounds.Exclamation.Play();
        } // button1_Click

        //static DataTable InxOp47()
        //{
        //    var dt = new DataTable();

        //    dt.Columns.Add("InstanceID", typeof(Byte));
        //    dt.Columns.Add("DateTimeValue", typeof(DateTime));
        //    dt.Columns.Add("database_id", typeof(Int16));
        //    dt.Columns.Add("object_id", typeof(Int32));
        //    dt.Columns.Add("index_id", typeof(Int32));
        //    dt.Columns.Add("partition_number", typeof(Int32));
        //    dt.Columns.Add("leaf_insert_count", typeof(Int64));
        //    dt.Columns.Add("leaf_delete_count", typeof(Int64));
        //    dt.Columns.Add("leaf_update_count", typeof(Int64));
        //    dt.Columns.Add("leaf_ghost_count", typeof(Int64));
        //    dt.Columns.Add("nonleaf_insert_count", typeof(Int64));
        //    dt.Columns.Add("nonleaf_delete_count", typeof(Int64));
        //    dt.Columns.Add("nonleaf_update_count", typeof(Int64));
        //    dt.Columns.Add("leaf_allocation_count", typeof(Int64));
        //    dt.Columns.Add("nonleaf_allocation_count", typeof(Int64));
        //    dt.Columns.Add("leaf_page_merge_count", typeof(Int64));
        //    dt.Columns.Add("nonleaf_page_merge_count", typeof(Int64));
        //    dt.Columns.Add("range_scan_count", typeof(Int64));
        //    dt.Columns.Add("singleton_lookup_count", typeof(Int64));
        //    dt.Columns.Add("forwarded_fetch_count", typeof(Int64));
        //    dt.Columns.Add("lob_fetch_in_pages", typeof(Int64));
        //    dt.Columns.Add("lob_fetch_in_bytes", typeof(Int64));
        //    dt.Columns.Add("lob_orphan_create_count", typeof(Int64));
        //    dt.Columns.Add("lob_orphan_insert_count", typeof(Int64));
        //    dt.Columns.Add("row_overflow_fetch_in_pages", typeof(Int64));
        //    dt.Columns.Add("row_overflow_fetch_in_bytes", typeof(Int64));
        //    dt.Columns.Add("column_value_push_off_row_count", typeof(Int64));
        //    dt.Columns.Add("column_value_pull_in_row_count", typeof(Int64));
        //    dt.Columns.Add("row_lock_count", typeof(Int64));
        //    dt.Columns.Add("row_lock_wait_count", typeof(Int64));
        //    dt.Columns.Add("row_lock_wait_in_ms", typeof(Int64));
        //    dt.Columns.Add("page_lock_count", typeof(Int64));
        //    dt.Columns.Add("page_lock_wait_count", typeof(Int64));
        //    dt.Columns.Add("page_lock_wait_in_ms", typeof(Int64));
        //    dt.Columns.Add("index_lock_promotion_attempt_count", typeof(Int64));
        //    dt.Columns.Add("index_lock_promotion_count", typeof(Int64));
        //    dt.Columns.Add("page_latch_wait_count", typeof(Int64));
        //    dt.Columns.Add("page_latch_wait_in_ms", typeof(Int64));
        //    dt.Columns.Add("page_io_latch_wait_count", typeof(Int64));
        //    dt.Columns.Add("page_io_latch_wait_in_ms", typeof(Int64));
        //    dt.Columns.Add("tree_page_latch_wait_count", typeof(Int64));
        //    dt.Columns.Add("tree_page_latch_wait_in_ms", typeof(Int64));
        //    dt.Columns.Add("tree_page_io_latch_wait_count", typeof(Int64));
        //    dt.Columns.Add("tree_page_io_latch_wait_in_ms", typeof(Int64));
        //    dt.Columns.Add("page_compression_attempt_count", typeof(Int64));
        //    dt.Columns.Add("page_compression_success_count", typeof(Int64));
        //    dt.Columns.Add("hobt_id", typeof(Int64));
        //    return dt;
        //}

        private void LoadFileUsingFieldTypesDataTable(String strFilenameWithPrefix, String strFilename)
        {
            DataTable csvData = new DataTable();
            csvData.TableName = "csvDataTable";
            String strFullFilename;
            Stopwatch swInterval;
            double duration;

            strFullFilename = strcFileDir + strFilenameWithPrefix + ".csv";
            lblFilename.Text = strFullFilename;

            swInterval = Stopwatch.StartNew();
            using (TextFieldParser csvReader = new TextFieldParser(strFullFilename))
            {
                csvReader.SetDelimiters(new string[] { "|" });
                csvReader.HasFieldsEnclosedInQuotes = false;
                string[] colFields = csvReader.ReadFields();

                // this code replaces the foreach loop in the string versions and tailors the 
                DataColumn dataColumn;
                dataColumn = new DataColumn("InstanceID", typeof(Byte));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("DateTimeValue", typeof(DateTime));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("database_id", typeof(Int16));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("object_id", typeof(Int32));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("index_id", typeof(Int32));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("partition_number", typeof(Int32));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("leaf_insert_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("leaf_delete_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("leaf_update_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("leaf_ghost_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("nonleaf_insert_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("nonleaf_delete_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("nonleaf_update_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("leaf_allocation_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("nonleaf_allocation_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("leaf_page_merge_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("nonleaf_page_merge_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("range_scan_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("singleton_lookup_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("forwarded_fetch_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("lob_fetch_in_pages", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("lob_fetch_in_bytes", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("lob_orphan_create_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("lob_orphan_insert_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("row_overflow_fetch_in_pages", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("row_overflow_fetch_in_bytes", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("column_value_push_off_row_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("column_value_pull_in_row_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("row_lock_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("row_lock_wait_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("row_lock_wait_in_ms", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_lock_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_lock_wait_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_lock_wait_in_ms", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("index_lock_promotion_attempt_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("index_lock_promotion_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_latch_wait_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_latch_wait_in_ms", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_io_latch_wait_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_io_latch_wait_in_ms", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("tree_page_latch_wait_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("tree_page_latch_wait_in_ms", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("tree_page_io_latch_wait_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("tree_page_io_latch_wait_in_ms", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_compression_attempt_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("page_compression_success_count", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);
                dataColumn = new DataColumn("hobt_id", typeof(Int64));
                dataColumn.AllowDBNull = true;
                csvData.Columns.Add(dataColumn);

                while (!csvReader.EndOfData)
                {
                    string[] fieldData = csvReader.ReadFields();

                    for (int i = 0; i < fieldData.Length; i++)
                    {
                        fieldData[i] = fieldData[i].Trim();
                    }
                    csvData.Rows.Add(fieldData);
                }
            }
            recordCount = csvData.Rows.Count;
            swInterval.Stop();
            duration = swInterval.Elapsed.TotalMilliseconds / 1000.0;
            logFile.WriteLine("{0},{1},{2},{3},{4}", strDesc, recordCount, "File", duration.ToString("#0.000"), recordCount / duration); //duration.ToString("HH:mm:ss.ffffff"));

            swInterval = Stopwatch.StartNew();

            SendTVP(csvData, recordCount);

            swInterval.Stop();
            duration = swInterval.Elapsed.TotalMilliseconds / 1000.0;
            logFile.WriteLine("{0},{1},{2},{3},{4}", strDesc, recordCount, "LTVP", duration.ToString("#0.000"), recordCount / duration); //duration.ToString("HH:mm:ss.ffffff"));
            csvData.Clear();
            csvData.Dispose();
        } // LoadFileUsingFieldTypesDataTable

        private void LoadFileUsingStringDataTable(String strFilenameWithPrefix, String strFilename)
        {
            DataTable csvData = new DataTable();
            csvData.TableName = "csvDataTable";
            String strFullFilename;
            Stopwatch swInterval;
            double duration;

            strFullFilename = strcFileDir + strFilenameWithPrefix + ".csv";
            lblFilename.Text = strFullFilename;

            swInterval = Stopwatch.StartNew();
            using (TextFieldParser csvReader = new TextFieldParser(strFullFilename))
            {
                csvReader.SetDelimiters(new string[] { "|" });
                csvReader.HasFieldsEnclosedInQuotes = false;
                string[] colFields = csvReader.ReadFields();

                foreach (string column in colFields)
                {
                    DataColumn dataColumn = new DataColumn(column);
                    dataColumn.AllowDBNull = true;
                    csvData.Columns.Add(dataColumn);
                }
                while (!csvReader.EndOfData)
                {
                    string[] fieldData = csvReader.ReadFields();

                    for (int i = 0; i < fieldData.Length; i++)
                    {
                        fieldData[i] = fieldData[i].Trim();
                    }
                    csvData.Rows.Add(fieldData);
                }
            }
            recordCount = csvData.Rows.Count;
            swInterval.Stop();
            duration = swInterval.Elapsed.TotalMilliseconds / 1000.0;
            logFile.WriteLine("{0},{1},{2},{3},{4}", strDesc, recordCount, "File", duration.ToString("#0.000"), recordCount / duration); //duration.ToString("HH:mm:ss.ffffff"));

            swInterval = Stopwatch.StartNew();

            SendTVP(csvData, recordCount);

            swInterval.Stop();
            duration = swInterval.Elapsed.TotalMilliseconds / 1000.0;
            logFile.WriteLine("{0},{1},{2},{3},{4}", strDesc, recordCount, "LTVP", duration.ToString("#0.000"), recordCount / duration); //duration.ToString("HH:mm:ss.ffffff"));
            csvData.Clear();
            csvData.Dispose();
        } // LoadFileUsingStringDataTable

        private void LoadFileUsingStringMaxLenDataTable(String strFilenameWithPrefix, String strFilename, int iMaxStringLen)
        {
            DataTable csvData = new DataTable();
            csvData.TableName = "csvDataTable";
            String strFullFilename;
            Stopwatch swInterval;
            double duration;

            strFullFilename = strcFileDir + strFilenameWithPrefix + ".csv";
            lblFilename.Text = strFullFilename;

            swInterval = Stopwatch.StartNew();
            using (TextFieldParser csvReader = new TextFieldParser(strFullFilename))
            {
                csvReader.SetDelimiters(new string[] { "|" });
                csvReader.HasFieldsEnclosedInQuotes = false;
                string[] colFields = csvReader.ReadFields();

                foreach (string column in colFields)
                {
                    DataColumn dataColumn = new DataColumn(column);
                    dataColumn.AllowDBNull = true;
                    dataColumn.MaxLength = iMaxStringLen;
                    csvData.Columns.Add(dataColumn);
                }
                while (!csvReader.EndOfData)
                {
                    string[] fieldData = csvReader.ReadFields();

                    for (int i = 0; i < fieldData.Length; i++)
                    {
                        fieldData[i] = fieldData[i].Trim();
                    }
                    csvData.Rows.Add(fieldData);
                }
            }
            recordCount = csvData.Rows.Count;
            swInterval.Stop();
            duration = swInterval.Elapsed.TotalMilliseconds / 1000.0;
            logFile.WriteLine("{0},{1},{2},{3},{4}", strDesc, recordCount, "File", duration.ToString("#0.000"), recordCount / duration); //duration.ToString("HH:mm:ss.ffffff"));

            swInterval = Stopwatch.StartNew();

            SendTVP(csvData, recordCount);

            swInterval.Stop();
            duration = swInterval.Elapsed.TotalMilliseconds / 1000.0;
            logFile.WriteLine("{0},{1},{2},{3},{4}", strDesc, recordCount, "LTVP", duration.ToString("#0.000"), recordCount / duration); //duration.ToString("HH:mm:ss.ffffff"));
            csvData.Clear();
            csvData.Dispose();
        } // LoadFileUsingStringMaxLenDataTable

        static int SendTVP(DataTable dt, int recordCount)
        {
            String strServer = "lenovo-jas"; //"Stx3-Desktopx64";
            String strDBName = "TestTVP";
            const String strcIntegratedSecurity = ";Integrated Security=SSPI";
            const String strcConnectionKeywords = ";Persist Security Info=True;Encrypt=False;Enlist=false;";
            String strConnectionKeywords = strcIntegratedSecurity + strcConnectionKeywords;
            const String strcPacketSize = "4096";
            int iTimeout = 1800;

            String strConnectionString =
                    "Server=" + strServer + strConnectionKeywords + "Initial Catalog=" + strDBName + ";Workstation ID=" + System.Windows.Forms.SystemInformation.ComputerName + "_TVPTest" + ";Packet Size=" + strcPacketSize + ";Pooling = false";

            using (SqlConnection conn = new SqlConnection(strConnectionString))
            {
                using (SqlCommand cmd = conn.CreateCommand())
                {
                    conn.Open();

                    cmd.Connection = conn;
                    cmd.CommandText = "truncate table SQLDynamicIndexOPStats";
                    cmd.CommandType = CommandType.Text;
                    cmd.CommandTimeout = 60; //seconds

                    cmd.ExecuteNonQuery();

                    cmd.CommandText = "ProcessAndInsertTVP";
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandTimeout = iTimeout; //seconds

                    SqlParameter InxOp = new SqlParameter("@InxOp", dt);
                    InxOp.SqlDbType = SqlDbType.Structured;
                    cmd.Parameters.Add(InxOp);
                    try
                    {
                        cmd.ExecuteNonQuery();
                    }
                    catch (SqlException e)
                    {
                        throw;
                    }
                    finally
                    {
                        conn.Close();
                        conn.Dispose();
                    }
                    return 1;
                } // command
            } // connection
        } // SendTVP
    } // class InitialForm 
} // namespace TestTVP
